
import os
from cement.core.foundation import CementApp
from cement.core.exc import FrameworkError, CaughtSignal
from cement.utils import fs, misc
from jpipes.core import exc

defaults = misc.init_defaults('jpipes', 'jenkins')
defaults['jenkins']['url'] = 'https://jenkins.example.com'
defaults['jenkins']['username'] = None
defaults['jenkins']['password'] = None
defaults['jenkins']['prefix'] = 'pipeline-'

class JPipesApp(CementApp):
    class Meta:
        label = 'jpipes'
        bootstrap = 'jpipes.cli.bootstrap'
        config_defaults = defaults
        extensions = ['yaml', 'colorlog']
        config_handler = 'yaml'
        log_handler = 'colorlog'
        config_files = [
            os.path.join('/', 'etc', label, 'jpipes.yml'),
            os.path.join(fs.HOME_DIR, '.jpipes.yml'),
            os.path.join(fs.HOME_DIR, '.jpipes', 'config'),
            os.path.join('jenkins', 'jpipes.yml')
        ]
        enable_framework_logging = False

class JPipesTestApp(JPipesApp):
    """A test app that is better suited for testing."""
    class Meta:
        argv = []
        config_files = []
        
def main():
    app = JPipesApp()
    try:
        with app:
            app.run()
    except CaughtSignal as e:
        app.log.warning(e)
        app.exit_code = 0
    except exc.JPipesError as e:
        app.log.fatal(e)
        app.exit_code = 1
    except FrameworkError as e:
        app.log.fatal(e)
        app.exit_code = 1
    finally:
        app.close()
    
if __name__ == '__main__':
    main()



import sys
from time import sleep
from cement.ext.ext_argparse import ArgparseController, expose as ex


class BaseController(ArgparseController):
    class Meta:
        label = 'base'
        description = 'Jenkins Pipeline Automation'
        default_func = '_default'

    @ex(hide=True)
    def _default(self):
        self.app.log.warning('Sub-Command required!')
        self.app.args.print_help()


class PipelineController(ArgparseController):
    class Meta:
        label = None
        help = 'pipeline sub-controller'

    def __init__(self, *args, **kw):
        super(PipelineController, self).__init__(*args, **kw)




import re
import requests


class JenkinsAPI(object):
    def __init__(self, base, auth=None, debug=False):
        self.base = base
        self.session = requests.Session()
        self.debug = debug
        if auth is not None:
            self.session.auth = auth

    def get(self, path):
        path = re.sub(self.base, '', path)
        if self.debug is True:
            print("GET: %s/%s" % (self.base, path))
        return self.session.get("%s/%s" % (self.base, path))

    def post(self, path, data={}):
        path = re.sub(self.base, '', path)
        if self.debug is True:
            print("POST: %s/%s - data:%s" % (self.base, path, data))
        return self.session.post("%s/%s" % (self.base, path), data=data)




import re
from cement.core import handler
from jpipes.cli.controllers.base import BaseController
from jpipes.cli.controllers.pipeline import PipelineController
from jpipes.core.jenkins import JenkinsAPI

def extend_jenkins_api(app):
    jenkins = JenkinsAPI(
                app.config.get('jenkins', 'url'), 
                auth=(
                    app.config.get('jenkins', 'username'), 
                    app.config.get('jenkins', 'password')
                ),
                debug=app.debug,
            )
    app.extend('api', jenkins)

def load_pipeline_controllers(app):
    pipelines = []

    # get all jobs that start with `prefix`
    prefix = app.config.get('jenkins', 'prefix')
    res = app.api.get('/api/json').json()
    for job in res['jobs']:
        if job['name'].startswith(prefix):
            pipelines.append(job['name'])

    for job in pipelines:
        pipe = {}
        pipe['label'] = re.sub(prefix, '', job)
        pipe['job'] = job
        pipe['arguments'] = []
        pipe['param_keys'] = []
        
        # get params for parameterized builds
        job_res = app.api.get('/job/%s/api/json' % pipe['job']).json()
        pipe['params'] = []
        param_class = 'hudson.model.ParametersDefinitionProperty'
        param_defs = []
        for _property in job_res['property']:
            if _property['_class'] == param_class:
                param_defs = _property['parameterDefinitions']
                break
        for param_def in param_defs:
            param = {}
            if param_def['_class'] == 'hudson.model.StringParameterDefinition':
                param['key'] = param_def['name']
                param['help'] = param_def['description']
                param['default'] = param_def['defaultParameterValue']['value']
            else:
                raise exc.JPipesError(
                    "Unsupported Paramater Type: %s" % param_def['_class']
                )

            pipe['params'].append(param)

        for param in pipe['params']:
            if isinstance(param, dict):
                key = param['key']
                
                # defaults
                param_dict = {
                    'name' : key,
                    'help' : '%s parameter' % key,
                    'default' : None,
                    'required' : True,
                }

                # override with what's passed
                param_dict.update(param)

            elif isinstance(param, str):
                # defaults
                param_dict = {
                    'key' : param,
                    'help' : '%s parameter' % param,
                    'default' : None,
                    'required' : True,
                }

            param = param_dict
            argument = ( [ '--%s' % param['key'] ], 
                         { 'help' : param['help'],
                           'dest' : param['key'],
                           'required' : param['required'] } )
            pipe['arguments'].append(argument)
            pipe['param_keys'].append(param['key'])

        class PipelineControllerInstance(PipelineController):
            class Meta:
                label = pipe['label']
                job = pipe['job']
                arguments = pipe['arguments']
                param_keys = pipe['param_keys']


        PipelineControllerInstance.__name__ = "%sPipelineController" % \
                                                pipe['label'].capitalize()
        app.handler.register(PipelineControllerInstance)

def load(app):
    app.handler.register(BaseController)
    app.hook.register('pre_run', extend_jenkins_api, weight=0)
    app.hook.register('pre_run', load_pipeline_controllers, weight=1)



"""Jpipes exception classes."""

class JPipesError(Exception):
    """Generic errors."""
    def __init__(self, msg):
        Exception.__init__(self)
        self.msg = msg
    
    def __str__(self):
        return self.msg

class JPipesConfigError(JPipesError):
    """Config related errors."""
    pass

class JPipesRuntimeError(JPipesError):
    """Generic runtime errors."""
    pass
        
class JPipesArgumentError(JPipesError):
    """Argument related errors."""
    pass




import re
import sys
from time import sleep
from cement.ext.ext_argparse import ArgparseController, expose as ex
from jpipes.core import exc


class PipelineController(ArgparseController):
    class Meta:
        label = None
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'pipeline sub-controller'
        default_func = '_default'
        console_log_poll_interval = 1
        job = None
        arguments = []
        param_keys = []

    def __init__(self, *args, **kw):
        super(PipelineController, self).__init__(*args, **kw)

    @ex(hide=True)
    def _default(self):    
        params = {}
        for param in self._meta.param_keys:
            if getattr(self.app.pargs, param) is not None:
                params[param] = getattr(self.app.pargs, param)

        if len(params) > 0:
            build_res = self.app.api.post('/job/%s/buildWithParameters' % \
                                          self._meta.job, params)
        else:
            build_res = self.app.api.post('/job/%s/build' % self._meta.job)

        assert build_res.status_code == 201, \
            "Unexpected Status Code: %s" % build_res.status_code

        self.app.log.info('WAITING FOR JOB TO START: %s/api/json' % \
                          build_res.headers['Location'].rstrip('/'))
        while True:
            path = "%s/api/json" % build_res.headers['Location'].rstrip('/')
            q_res = self.app.api.get(path).json()
            # odd but at times can hit a little too early and the data isn't
            # populated in the api
            if 'executable' in q_res.keys() \
                and q_res['executable'] is not None:
                if 'url' in q_res['executable'].keys() \
                    and q_res['executable']['url'] is not None:
                    break
            else:
                sleep(1)

        self.app.log.info("JOB STARTED: %s" % q_res['executable']['url'])

        offset = 0
        while True:
            log_url = "%s/logText/progressiveText?start=%s" % (
                        q_res['executable']['url'],
                        offset
                        )
            log_res = self.app.api.get(log_url)
            
            if len(log_res.text):
                print(log_res.text)

            if 'X-More-Data' in log_res.headers:
                offset = log_res.headers['X-Text-Size']
                sleep(self._meta.console_log_poll_interval)
            else:
                break

        path = "%s/api/json" % q_res['executable']['url']
        job_res = self.app.api.get(path).json()
        if job_res['result'] == 'SUCCESS': 
            self.app.log.info('JOB COMPLETED SUCCESSFULLY')
        else:
            raise exc.JPipesError("JOB FAILED! %s" % \
                                  q_res['executable']['url'])
